@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-certificate.cpp,v 1.1.4.1 2004/06/23 21:22:13 chpe Exp $
+ *  $Id: mozilla-embed-certificate.cpp,v 1.2 2004/05/19 18:12:45 chpe Exp $
  */
 
 #include "mozilla-embed-certificate.h"
@@ -25,30 +25,34 @@
 #include <nsCOMPtr.h>
 #include <nsIRequest.h>
 
-#define DATA_KEY "EphyCertificateExtension::MozillaEmbedCertificate"
+#define DATA_KEY "EphyCertificatesExtension::MozillaEmbedCertificate"
 
-static void embed_security_change_cb (GObject *embed_object, 
-				      gpointer request_ptr,
-				      guint state,
-				      MozillaEmbedCertificate *cert)
+static void
+embed_security_change_cb (GObject *embed_object, 
+			  gpointer request_ptr,
+			  guint state,
+			  gpointer dummy)
 {
+	MozillaEmbedCertificate *cert;
+
+	cert = (MozillaEmbedCertificate *) g_object_get_data (embed_object, DATA_KEY);
+	g_return_if_fail (cert);
 	if (!cert) return;
 
 	nsCOMPtr<nsIRequest> request = static_cast<nsIRequest*>(request_ptr);
-	NS_ENSURE_TRUE (request,);
+	g_return_if_fail (request != nsnull);
+	if (!request) return;
 
 	cert->SetCertificateFromRequest (request);
 }
 
 static void
-embed_destroy_cb (GtkObject *object,
-		  MozillaEmbedCertificate *cert)
+delete_cert (gpointer cert_ptr)
 {
-	g_return_if_fail (cert != NULL);
+	MozillaEmbedCertificate *cert = (MozillaEmbedCertificate *) cert_ptr;
 
-	g_signal_handlers_disconnect_by_func
-		(object, (void*) embed_security_change_cb, cert);
-	g_object_set_data (G_OBJECT (object), DATA_KEY, NULL);
+	g_return_if_fail (cert);
+	if (!cert) return;
 
 	delete cert;
 }
@@ -64,13 +68,12 @@ mozilla_embed_certificate_attach (EphyEmbed *embed)
 	{
 		cert = new MozillaEmbedCertificate ();
 
-		g_object_set_data (embed_object, DATA_KEY, cert);
+		g_object_set_data_full (embed_object, DATA_KEY, cert,
+					(GDestroyNotify) delete_cert);
 
-		g_signal_connect (embed_object, "destroy",
-				  G_CALLBACK (embed_destroy_cb), cert);
-
-		g_signal_connect (G_OBJECT (embed), "security_change",
-				  G_CALLBACK (embed_security_change_cb), cert);
+		g_signal_connect_object (G_OBJECT (embed), "security_change",
+					 G_CALLBACK (embed_security_change_cb),
+					 NULL, (GConnectFlags) 0);
 	}
 }
 
@@ -81,6 +84,7 @@ mozilla_embed_view_certificate (EphyEmbed *embed)
 	MozillaEmbedCertificate *cert;
 
 	cert = (MozillaEmbedCertificate *) g_object_get_data (embed_object, DATA_KEY);
+	g_return_if_fail (cert);
 	if (!cert) return;
 
 	cert->ViewCertificate ();
@@ -93,6 +97,7 @@ mozilla_embed_has_certificate (EphyEmbed *embed)
 	MozillaEmbedCertificate *cert;
 
 	cert = (MozillaEmbedCertificate *) g_object_get_data (embed_object, DATA_KEY);
+	g_return_val_if_fail (cert, FALSE);
 	if (!cert) return FALSE;
 
 	PRBool hasCert;
