@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-helpers.cpp,v 1.5 2004/03/15 22:07:05 adamh Exp $
+ *  $Id: mozilla-helpers.cpp,v 1.7 2004/05/07 14:34:27 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -65,9 +65,13 @@ mozilla_register_popup_listener (EphyEmbed *embed)
 	nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface (domWindow, &rv));
 	NS_ENSURE_SUCCESS (rv, NULL);
 
+#if MOZILLA_SNAPSHOT >= 18
+	nsIChromeEventHandler* chromeHandler;
+	chromeHandler = piWin->GetChromeEventHandler ();
+#else
 	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-	rv = piWin->GetChromeEventHandler (getter_AddRefs (chromeHandler));
-	NS_ENSURE_SUCCESS (rv, NULL);
+	piWin->GetChromeEventHandler (getter_AddRefs (chromeHandler));
+#endif
 
 	nsCOMPtr<nsIDOMEventReceiver> eventReceiver;
 	eventReceiver = do_QueryInterface (chromeHandler, &rv);
